/////////////////////////////////////////////////////
function [out, func, err] = quadratique_approx(in, lambda, nbIter)
//
//       function [out, func, err] = quadratique_approx(in, lambda, nbIter)
//
//
//  in : image en entree
//  lambda : parametre de regularisation
//  nbIter : nombre d'itérations
//  out : image en sortie. Elle est une solution approchee de
//        out  = argmin_u ||grad u||^2 + lambda || u - in ||^2
//  func : valeur de la fonctionnelle a la fin de chaque iteration
//  err : norme de l'erreur entre la solution exacte et le resultat de l'iteration
//
//

out_exact = quadratique_exacte(in, lambda);
out = in;
for iter = 1:nbIter
	descente = moins_gradient(in,lambda,out);
	[step, func(iter)] = deepest_step(in,lambda,out,descente);
	out = out + step * descente;
	err(iter) = norm(out - out_exact);	

	printf('iteration %d,      step %g,       functional value %g,      distance to target %g\n',iter,step,func(iter),err(iter));
end


endfunction;

//////////////////////////////////////////////////////////////////////////////
function [step, value] = deepest_step(in,lambda,out,direction)

deg0=0;deg1=0;deg2=0;
out_x = grad_x(out,0);
out_y = grad_y(out,0);
direction_x = grad_x(direction,0);
direction_y = grad_y(direction,0);

deg0 = sum(sum( out_x.*out_x ))+sum(sum( out_y.*out_y )) + lambda*sum(sum( (out-in).*(out-in) ));
deg1 = sum(sum( out_x.*direction_x ))+sum(sum( out_y.*direction_y )) + lambda*sum(sum( (out-in).*direction ));
deg1 = 2*deg1;
deg2 = sum(sum( direction_x.*direction_x ))+sum(sum( direction_y.*direction_y )) + lambda*sum(sum( direction.*direction ));

if deg2==0
	step = 0;
else
	step = -deg1/(2*deg2);
end 

value = deg2*step*step + deg1*step + deg0 ;

endfunction;

//////////////////////////////////////////////////////////////////////////////
function descente = moins_gradient(in,lambda,out)

tmp = grad_x(out,0);
lap_x= grad_x(tmp,1);
tmp = grad_y(out,0);
lap_y= grad_y(tmp,1);

descente = - (2*lap_x +2* lap_y + 2*lambda*(out-in));

endfunction;

//////////////////////////////////////////////////////////////////////////////
function out = grad_x(in,adjoint)

[sx,sy]=size(in);

if adjoint == 0
	tmp = [in(2:sx,:) ; in(1,:)];
	out = tmp - in;
else
	tmp = [ in(sx,:) ; in(1:sx-1,:) ];
	out = tmp - in;
end

endfunction;
//////////////////////////////////////////////////////////////////////////////
function out = grad_y(in,adjoint)

[sx,sy]=size(in);

if adjoint == 0
	tmp = [in(:,2:sy) , in(:,1)];
	out = tmp - in;
else
	tmp = [ in(:,sy) , in(:,1:sy-1) ];
	out = tmp - in;
end

endfunction;
//////////////////////////////////////////////////////////////////////////////








